local DamageType = require "engine.DamageType"
local ActorInventory = require "engine.interface.ActorInventory"
-- Made by Nuku Valente(David Silver)
-- Copyright 2014
-- Designed For:
-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009, 2010, 2011 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

newBirthDescriptor{
	type = "race",
	name = "Furrae",
	desc = {
		"The Furred Folk.",
	},
	descriptor_choices =
	{
		subrace =
		{
			__ALL__ = "disallow",
			Feline = "allow",
			["Grey Canine"] = "allow",
			["Hooter"] = "allow",
			["Blackback"] = "allow",
			["Equis"] = "allow",
		},
		class =
		{
--			__ALL__ = "allow",
--			None = "disallow",
		},
	},
	copy = {
		type = "humanoid", subtype="Fey",
		starting_zone = "trollmire",
		starting_quest = "start-allied",
	},
}

newBirthDescriptor
{
	type = "subrace",
	name = "Feline",
	desc = {
		"Felines are lithe and agile member of the furrae races.",
		"They are agile and make use of their #GOLD#speed#WHITE# to maneuver and outwit their foes.",
		"Some say cats have nine lives, but there seems little truth in it.",
		"Having oddly shaped feet, they do not wear boots, but can fit an extra ring on their tail.",
		"#GOLD#Stat modifiers:",
		"#LIGHT_BLUE# * -1 Strength, +3 Dexterity, +0 Constitution",
		"#LIGHT_BLUE# * -3 Magic, -2 Willpower, +3 Cunning",
		"#LIGHT_BLUE# * +10 Luck",
		"#GOLD#Life per level:#LIGHT_BLUE# 10",
		"#GOLD#Experience penalty:#LIGHT_BLUE# +25%",
	},
	inc_stats = { str=-1, con=0, mag=-3, dex=3, cun=3, wil=-2, lck=10, },
	talents_types = {
		["race/feline"]={true, 0},
	},
	copy = {
		moddable_tile = "feline",
		moddable_tile_base = "base_feline_01.png",
		--moddable_tile_ornament = {female="braid_02"},
		random_name_def = "shalore_#sex#", random_name_max_syllables = 7,
		default_wilderness = {"playerpop", "allied"},
		starting_zone = "trollmire",
		starting_quest = "start-allied",
		faction = "thalore",
		starting_intro = "furrae",
		life_rating = 10,
		infravision = 2,
		equipdoll = "feline",
		resists = { [DamageType.FIRE] = -5, [DamageType.COLD] = 10, },
		resolvers.inscription("RUNE:_SHIELDING", {cooldown=10, dur=5, power=100}),
		resolvers.inscription("INFUSION:_REGENERATION", {cooldown=10, dur=5, heal=60}),
		resolvers.inventory{ id=true, {defined="ORB_SCRYING"} },
		resolvers.generic(function(e) e.inven[e.INVEN_FEET] = nil end)
	},
	talents_types = { ["race/feline"]={true, 0} },
	talents = {
		[ActorTalents.T_SUDDEN_DASH]=1,
	},
	body = {FINGER = 3,LITE=1,HEAD=1,NECK=1,TOOL=1,MAINHAND=1,OFFHAND=1,FEET="0",BODY=1,QUIVER=1,BELT=1,HANDS=1,CLOAK=1},
	experience = 1.25,
	random_escort_possibilities = { {"tier1.1", 1, 2}, {"tier1.2", 1, 2}, {"daikara", 1, 2}, {"old-forest", 1, 4}, {"dreadfell", 1, 8}, {"reknor", 1, 2}, },
}

newBirthDescriptor
{
type = "subrace",
	name = "Grey Canine",
	desc = {
		"The grey wolves, or canines as they prefer, fancy themselves the leaders of the furrae races.",
		"They have a natural #GOLD#empathy#WHITE# with canines and are only rarely seen without one.",
		"Proud and tough, grey canines that adventure out into the world do so with confidence.",
		"Having oddly shaped feet, they do not wear boots. Unlike showy felines, they do not ornament their tails.",
		"#GOLD#Stat modifiers:",
		"#LIGHT_BLUE# * +2 Strength, +0 Dexterity, +2 Constitution",
		"#LIGHT_BLUE# * -3 Magic, +2 Willpower, -2 Cunning",
		"#GOLD#Life per level:#LIGHT_BLUE# 11",
		"#GOLD#Experience penalty:#LIGHT_BLUE# +15%",
	},
	inc_stats = { str=2, con=2, mag=-3, dex=0, cun=-2, wil=2, },
	talents_types = {
		["race/gcanine"]={true, 0},
	},
	copy = {
		moddable_tile = "greycanine",
		moddable_tile_base = "base_canine_01.png",
		--moddable_tile_ornament = {female="braid_02"},
		random_name_def = "shalore_#sex#", random_name_max_syllables = 7,
		default_wilderness = {"playerpop", "allied"},
		starting_zone = "trollmire",
		starting_quest = "start-allied",
		faction = "thalore",
		starting_intro = "furrae",
		life_rating = 11,
		infravision = 2,
		equipdoll = "greycanine",
		resists = { [DamageType.FIRE] = -5, [DamageType.COLD] = 10, },
		resolvers.inscription("RUNE:_SHIELDING", {cooldown=10, dur=5, power=100}),
		resolvers.inscription("INFUSION:_REGENERATION", {cooldown=10, dur=5, heal=60}),
		resolvers.inventory{ id=true, {defined="ORB_SCRYING"} },
		resolvers.generic(function(e) e.inven[e.INVEN_FEET] = nil end)
	},
	talents_types = { ["race/gcanine"]={true, 0} },
	talents = {
		[ActorTalents.T_WOLF_ALLY]=1,
	},
	body = {FINGER = 2,LITE=1,HEAD=1,NECK=1,TOOL=1,MAINHAND=1,OFFHAND=1,FEET="0",BODY=1,QUIVER=1,BELT=1,HANDS=1,CLOAK=1},
	experience = 1.15,
	random_escort_possibilities = { {"tier1.1", 1, 2}, {"tier1.2", 1, 2}, {"daikara", 1, 2}, {"old-forest", 1, 4}, {"dreadfell", 1, 8}, {"reknor", 1, 2}, },
}

newBirthDescriptor
{
type = "subrace",
	name = "Hooter",
	desc = {
		"Hooters are a race of owl like beings, with large eyes that pierce the darkness and soft feathery wings.",
		"They have a #GOLD#wisdom#WHITE# that sees them through the hard times and guides them away from danger.",
		"Thoughtful and pensive, adventuring hooters are often in search of great answers, often to questions no one asked.",
		"Having oddly shaped feet, they do not wear boots. They do enjoy jewelry however. Having no long tail to hang them on, they simply hang another amulet around their neck.",
		"#GOLD#Stat modifiers:",
		"#LIGHT_BLUE# * -2 Strength, +0 Dexterity, -2 Constitution",
		"#LIGHT_BLUE# * +3 Magic, +1 Willpower, +3 Cunning",
		"#GOLD#Life per level:#LIGHT_BLUE# 9",
		"#GOLD#Experience penalty:#LIGHT_BLUE# +15%",
	},
	inc_stats = { str=-2, con=-2, mag=3, dex=0, cun=3, wil=1, },
	talents_types = {
		["race/hooter"]={true, 0},
	},
	copy = {
		moddable_tile = "hooter",
		moddable_tile_base = "base_hooter_01.png",
		--moddable_tile_ornament = {female="braid_02"},
		random_name_def = "shalore_#sex#", random_name_max_syllables = 7,
		default_wilderness = {"playerpop", "allied"},
		starting_zone = "trollmire",
		starting_quest = "start-allied",
		faction = "thalore",
		starting_intro = "furrae",
		life_rating = 9,
		infravision = 4,
		equipdoll = "hooter",
		resists = { [DamageType.FIRE] = -5, [DamageType.COLD] = 10, [DamageType.MIND] = 5},
		resolvers.inscription("RUNE:_SHIELDING", {cooldown=10, dur=5, power=100}),
		resolvers.inscription("INFUSION:_REGENERATION", {cooldown=10, dur=5, heal=60}),
		resolvers.inventory{ id=true, {defined="ORB_SCRYING"} },
		resolvers.generic(function(e) e.inven[e.INVEN_FEET] = nil end)
	},
	talents_types = { ["race/hooter"]={true, 0} },
	talents = {
		[ActorTalents.T_OBSERVE_THE_WORLD]=1,
	},
	body = {FINGER = 2,LITE=1,HEAD=1,NECK=2,TOOL=1,MAINHAND=1,OFFHAND=1,FEET="0",BODY=1,QUIVER=1,BELT=1,HANDS=1,CLOAK=1},
	experience = 1.15,
	random_escort_possibilities = { {"tier1.1", 1, 2}, {"tier1.2", 1, 2}, {"daikara", 1, 2}, {"old-forest", 1, 4}, {"dreadfell", 1, 8}, {"reknor", 1, 2}, },
}

newBirthDescriptor
{
type = "subrace",
	name = "Blackback",
	desc = {
		"Blackbacks are creatures of bold black and white fur, bright for warning of their toxins.",
		"They make use of this #GOLD#spray#WHITE# to dissuade agressors and are not known to back down in a fight.",
		"When not battling, Blackbacks are a sociable and warm people. They make great neighbors, if you don't mind the smell.",
		"Unlike most furrae species, they have feet compatible with the furless, allowing them to wear boots.",
		"#GOLD#Stat modifiers:",
		"#LIGHT_BLUE# * +1 Strength, +0 Dexterity, +1 Constitution",
		"#LIGHT_BLUE# * +0 Magic, +2 Willpower, +0 Cunning",
		"#GOLD#Life per level:#LIGHT_BLUE# 10",
		"#GOLD#Experience penalty:#LIGHT_BLUE# +15%",
	},
	inc_stats = { str=1, con=1, mag=0, dex=0, cun=0, wil=2, },
	talents_types = {
		["race/blackback"]={true, 0},
	},
	copy = {
		moddable_tile = "blackback",
		moddable_tile_base = "base_blackback_01.png",
		--moddable_tile_ornament = {female="braid_02"},
		random_name_def = "shalore_#sex#", random_name_max_syllables = 7,
		default_wilderness = {"playerpop", "allied"},
		starting_zone = "trollmire",
		starting_quest = "start-allied",
		faction = "thalore",
		starting_intro = "furrae",
		life_rating = 10,
		infravision = 2,
		poison_immune = 0.1,
		fear_immune = 0.2,
		resists = { 
			[DamageType.FIRE] = -5, 
			[DamageType.COLD] = 10, 
			[DamageType.NATURE] = 10, 
			[DamageType.BLIGHT] = 5,
		},
		resolvers.inscription("RUNE:_SHIELDING", {cooldown=10, dur=5, power=100}),
		resolvers.inscription("INFUSION:_REGENERATION", {cooldown=10, dur=5, heal=60}),
		resolvers.inventory{ id=true, {defined="ORB_SCRYING"} },
		},
	talents_types = { ["race/blackback"]={true, 0} },
	talents = {
		[ActorTalents.T_MUSK_SPRAY]=1,
	},
	body = {FINGER = 2,LITE=1,HEAD=1,NECK=1,TOOL=1,MAINHAND=1,OFFHAND=1,FEET=1,BODY=1,QUIVER=1,BELT=1,HANDS=1,CLOAK=1},
	experience = 1.15,
	random_escort_possibilities = { {"tier1.1", 1, 2}, {"tier1.2", 1, 2}, {"daikara", 1, 2}, {"old-forest", 1, 4}, {"dreadfell", 1, 8}, {"reknor", 1, 2}, },
}

newBirthDescriptor
{
type = "subrace",
	name = "Equis",
	desc = {
		"Imposingly large, Equis have the most striking resemblane for humans, if not for their horse like skulls, tails, and hooves.",
		"They make use of their #GOLD#strength#WHITE# to push aside smaller things in their way.",
		"Unlike their peaceful natural counterparts, Equis revel in battle, proving their worth against the steel and wit of their foes.",
		"Like must furrae, their feet are ill equipped for boots, but their trophy hunting habits leaves them quite ornamented.",
		"#GOLD#Stat modifiers:",
		"#LIGHT_BLUE# * +2 Strength, +0 Dexterity, +2 Constitution",
		"#LIGHT_BLUE# * -3 Magic, +1 Willpower, -1 Cunning",
		"#GOLD#Life per level:#LIGHT_BLUE# 12",
		"#GOLD#Experience penalty:#LIGHT_BLUE# +15%",
	},
	inc_stats = { str=2, con=2, mag=-3, dex=0, cun=-1, wil=1, },
	talents_types = {
		["race/equis"]={true, 0},
	},
	copy = {
		moddable_tile = "equis",
		moddable_tile_base = "base_equis.png",
		--moddable_tile_ornament = {female="braid_02"},
		random_name_def = "shalore_#sex#", random_name_max_syllables = 7,
		default_wilderness = {"playerpop", "allied"},
		starting_zone = "trollmire",
		starting_quest = "start-allied",
		faction = "thalore",
		starting_intro = "furrae",
		life_rating = 12,
		infravision = 2,
		fear_immune = 0.2,
		size_category = 5,
		equipdoll = "equis",
		resolvers.inscription("RUNE:_SHIELDING", {cooldown=10, dur=5, power=100}),
		resolvers.inscription("INFUSION:_REGENERATION", {cooldown=10, dur=5, heal=60}),
		resolvers.inventory{ id=true, {defined="ORB_SCRYING"} },
		},
	talents_types = { ["race/equis"]={true, 0} },
	talents = {
		[ActorTalents.T_BUCK]=1,
	},
	body = {FINGER = 3,LITE=1,HEAD=1,NECK=2,TOOL=1,MAINHAND=1,OFFHAND=1,FEET=nil,BODY=1,QUIVER=1,BELT=1,HANDS=1,CLOAK=1},
	experience = 1.15,
	random_escort_possibilities = { {"tier1.1", 1, 2}, {"tier1.2", 1, 2}, {"daikara", 1, 2}, {"old-forest", 1, 4}, {"dreadfell", 1, 8}, {"reknor", 1, 2}, },
}

getBirthDescriptor("world", "Maj'Eyal").descriptor_choices.race.Furrae = "allow"
getBirthDescriptor("world", "Infinite").descriptor_choices.race.Furrae = "allow"
getBirthDescriptor("world", "Arena").descriptor_choices.race.Furrae = "allow"

ActorInventory.equipdolls.hooter = { w=48, h=48, itemframe="ui/equipdoll/itemframe48.png", itemframe_sel="ui/equipdoll/itemframe-sel48.png", ix=3, iy=3, iw=42, ih=42, doll_x=116, doll_y=168+64, list={
		PSIONIC_FOCUS = {{weight=1, x=48, y=48}},
		MAINHAND = {{weight=2, x=48, y=120}},
		OFFHAND = {{weight=3, x=48, y=192}},
		FINGER = {{weight=6, x=48, y=408}, {weight=7, x=120, y=408, text="bottom"}},
		BODY = {{weight=4, x=48, y=264}},
		QUIVER = {{weight=5, x=48, y=336}},
		LITE = {{weight=9, x=192, y=408}},
		TOOL = {{weight=10, x=264, y=408, text="bottom"}},
		--FEET = {{weight=10, x=264, y=336}},
		BELT = {{weight=11, x=264, y=264}},
		HANDS = {{weight=12, x=264, y=192}},
		CLOAK = {{weight=13, x=264, y=120}},
		NECK = {{weight=14, x=192, y=48, text="topright"}, {weight=8, x=264, y=336 }},
		HEAD = {{weight=15, x=120, y=48, text="topleft"}},
}}

ActorInventory.equipdolls.feline = { w=48, h=48, itemframe="ui/equipdoll/itemframe48.png", itemframe_sel="ui/equipdoll/itemframe-sel48.png", ix=3, iy=3, iw=42, ih=42, doll_x=116, doll_y=168+64, list={
		PSIONIC_FOCUS = {{weight=1, x=48, y=48}},
		MAINHAND = {{weight=2, x=48, y=120}},
		OFFHAND = {{weight=3, x=48, y=192}},
		FINGER = {{weight=6, x=48, y=408}, {weight=7, x=120, y=408, text="bottom"}, {weight=8, x=264, y=336 }},
		BODY = {{weight=4, x=48, y=264}},
		QUIVER = {{weight=5, x=48, y=336}},
		LITE = {{weight=9, x=192, y=408}},
		TOOL = {{weight=10, x=264, y=408, text="bottom"}},
		--FEET = {{weight=10, x=264, y=336}},
		BELT = {{weight=11, x=264, y=264}},
		HANDS = {{weight=12, x=264, y=192}},
		CLOAK = {{weight=13, x=264, y=120}},
		NECK = {{weight=14, x=192, y=48, text="topright"}},
		HEAD = {{weight=15, x=120, y=48, text="topleft"}},
}}

ActorInventory.equipdolls.equis = { w=48, h=48, itemframe="ui/equipdoll/itemframe48.png", itemframe_sel="ui/equipdoll/itemframe-sel48.png", ix=3, iy=3, iw=42, ih=42, doll_x=116, doll_y=168+64, list={
		PSIONIC_FOCUS = {{weight=1, x=48, y=48}},
		MAINHAND = {{weight=2, x=48, y=120}},
		OFFHAND = {{weight=3, x=48, y=192}},
		FINGER = {{weight=6, x=48, y=408}, {weight=7, x=120, y=408, text="bottom"}, {weight=8, x=264, y=336 }},
		BODY = {{weight=4, x=48, y=264}},
		QUIVER = {{weight=5, x=48, y=336}},
		LITE = {{weight=9, x=192, y=408}},
		TOOL = {{weight=10, x=264, y=408, text="bottom"}},
		--FEET = {{weight=10, x=264, y=336}},
		BELT = {{weight=11, x=264, y=264}},
		HANDS = {{weight=12, x=264, y=192}},
		CLOAK = {{weight=13, x=264, y=120}},
		NECK = {{weight=14, x=192, y=48, text="topright"},{weight=14, x=192+60, y=48}},
		HEAD = {{weight=15, x=120, y=48, text="topleft"}},
}}

ActorInventory.equipdolls.animal = { w=48, h=48, itemframe="ui/equipdoll/itemframe48.png", itemframe_sel="ui/equipdoll/itemframe-sel48.png", ix=3, iy=3, iw=42, ih=42, doll_x=116, doll_y=168+64, list={
		BODY = {{weight=4, x=48, y=264}},
}}

ActorInventory.equipdolls.greycanine = { w=48, h=48, itemframe="ui/equipdoll/itemframe48.png", itemframe_sel="ui/equipdoll/itemframe-sel48.png", ix=3, iy=3, iw=42, ih=42, doll_x=116, doll_y=168+64, list={
		PSIONIC_FOCUS = {{weight=1, x=48, y=48}},
		MAINHAND = {{weight=2, x=48, y=120}},
		OFFHAND = {{weight=3, x=48, y=192}},
		FINGER = {{weight=6, x=48, y=408}, {weight=7, x=120, y=408, text="bottom"}},
		BODY = {{weight=4, x=48, y=264}},
		QUIVER = {{weight=5, x=48, y=336}},
		LITE = {{weight=9, x=192, y=408}},
		TOOL = {{weight=10, x=264, y=408, text="bottom"}},
		--FEET = {{weight=10, x=264, y=336}},
		BELT = {{weight=11, x=264, y=264}},
		HANDS = {{weight=12, x=264, y=192}},
		CLOAK = {{weight=13, x=264, y=120}},
		NECK = {{weight=14, x=192, y=48, text="topright"}},
		HEAD = {{weight=15, x=120, y=48, text="topleft"}},
}}