local ActorInventory = require "engine.interface.ActorInventory"
local Object = require "mod.class.Object"
local Chat = require("engine.Chat")
local Entity = require "engine.Entity"

damDesc = function(self, type, dam)
	-- Increases damage
	if self.inc_damage then
		local inc = (self.inc_damage.all or 0) + (self.inc_damage[type] or 0)
		dam = dam + (dam * inc / 100)
	end
	return dam
end

racial_req1 = {
	level = function(level) return 0 + (level-1)  end,
}
racial_req2 = {
	level = function(level) return 4 + (level-1)  end,
}
racial_req3 = {
	level = function(level) return 8 + (level-1)  end,
}
racial_req4 = {
	level = function(level) return 12 + (level-1)  end,
}
-- Generic requires for spells based on talent level
spells_req1 = {
	stat = { mag=function(level) return 12 + (level-1) * 2 end },
	level = function(level) return 0 + (level-1)  end,
}
spells_req2 = {
	stat = { mag=function(level) return 20 + (level-1) * 2 end },
	level = function(level) return 4 + (level-1)  end,
}
spells_req3 = {
	stat = { mag=function(level) return 28 + (level-1) * 2 end },
	level = function(level) return 8 + (level-1)  end,
}
spells_req4 = {
	stat = { mag=function(level) return 36 + (level-1) * 2 end },
	level = function(level) return 12 + (level-1)  end,
}
spells_req5 = {
	stat = { mag=function(level) return 44 + (level-1) * 2 end },
	level = function(level) return 16 + (level-1)  end,
}
spells_req_high1 = {
	stat = { mag=function(level) return 22 + (level-1) * 2 end },
	level = function(level) return 10 + (level-1)  end,
}
spells_req_high2 = {
	stat = { mag=function(level) return 30 + (level-1) * 2 end },
	level = function(level) return 14 + (level-1)  end,
}
spells_req_high3 = {
	stat = { mag=function(level) return 38 + (level-1) * 2 end },
	level = function(level) return 18 + (level-1)  end,
}
spells_req_high4 = {
	stat = { mag=function(level) return 46 + (level-1) * 2 end },
	level = function(level) return 22 + (level-1)  end,
}
spells_req_high5 = {
	stat = { mag=function(level) return 54 + (level-1) * 2 end },
	level = function(level) return 26 + (level-1)  end,
}
-- Generic requires for cunning based on talent level
cuns_req1 = {
	stat = { cun=function(level) return 12 + (level-1) * 2 end },
	level = function(level) return 0 + (level-1)  end,
}
cuns_req2 = {
	stat = { cun=function(level) return 20 + (level-1) * 2 end },
	level = function(level) return 4 + (level-1)  end,
}
cuns_req3 = {
	stat = { cun=function(level) return 28 + (level-1) * 2 end },
	level = function(level) return 8 + (level-1)  end,
}
cuns_req4 = {
	stat = { cun=function(level) return 36 + (level-1) * 2 end },
	level = function(level) return 12 + (level-1)  end,
}
cuns_req5 = {
	stat = { cun=function(level) return 44 + (level-1) * 2 end },
	level = function(level) return 16 + (level-1)  end,
}
cuns_req_high1 = {
	stat = { cun=function(level) return 22 + (level-1) * 2 end },
	level = function(level) return 10 + (level-1)  end,
}
cuns_req_high2 = {
	stat = { cun=function(level) return 30 + (level-1) * 2 end },
	level = function(level) return 14 + (level-1)  end,
}
cuns_req_high3 = {
	stat = { cun=function(level) return 38 + (level-1) * 2 end },
	level = function(level) return 18 + (level-1)  end,
}
cuns_req_high4 = {
	stat = { cun=function(level) return 46 + (level-1) * 2 end },
	level = function(level) return 22 + (level-1)  end,
}
cuns_req_high5 = {
	stat = { cun=function(level) return 54 + (level-1) * 2 end },
	level = function(level) return 26 + (level-1)  end,
}
-- Generic requires for techs based on talent level
-- Uses STR
techs_req1 = function(self, t) local stat = "str"; return {
	stat = { [stat]=function(level) return 12 + (level-1) * 2 end },
	level = function(level) return 0 + (level-1)  end,
} end
techs_req2 = function(self, t) local stat = "str"; return {
	stat = { [stat]=function(level) return 20 + (level-1) * 2 end },
	level = function(level) return 4 + (level-1)  end,
} end
techs_req3 = function(self, t) local stat = "str"; return {
	stat = { [stat]=function(level) return 28 + (level-1) * 2 end },
	level = function(level) return 8 + (level-1)  end,
} end
techs_req4 = function(self, t) local stat = "str"; return {
	stat = { [stat]=function(level) return 36 + (level-1) * 2 end },
	level = function(level) return 12 + (level-1)  end,
} end
techs_req5 = function(self, t) local stat = "str"; return {
	stat = { [stat]=function(level) return 44 + (level-1) * 2 end },
	level = function(level) return 16 + (level-1)  end,
} end
techs_req_high1 = function(self, t) local stat = "str"; return {
	stat = { [stat]=function(level) return 22 + (level-1) * 2 end },
	level = function(level) return 10 + (level-1)  end,
} end
techs_req_high2 = function(self, t) local stat = "str"; return {
	stat = { [stat]=function(level) return 30 + (level-1) * 2 end },
	level = function(level) return 14 + (level-1)  end,
} end
techs_req_high3 = function(self, t) local stat = "str"; return {
	stat = { [stat]=function(level) return 38 + (level-1) * 2 end },
	level = function(level) return 18 + (level-1)  end,
} end
techs_req_high4 = function(self, t) local stat = "str"; return {
	stat = { [stat]=function(level) return 46 + (level-1) * 2 end },
	level = function(level) return 22 + (level-1)  end,
} end
techs_req_high5 = function(self, t) local stat = "str"; return {
	stat = { [stat]=function(level) return 54 + (level-1) * 2 end },
	level = function(level) return 26 + (level-1)  end,
} end
-- Generic requires for gifts based on talent level
gifts_req1 = {
	stat = { wil=function(level) return 12 + (level-1) * 2 end },
	level = function(level) return 0 + (level-1)  end,
}
gifts_req2 = {
	stat = { wil=function(level) return 20 + (level-1) * 2 end },
	level = function(level) return 4 + (level-1)  end,
}
gifts_req3 = {
	stat = { wil=function(level) return 28 + (level-1) * 2 end },
	level = function(level) return 8 + (level-1)  end,
}
gifts_req4 = {
	stat = { wil=function(level) return 36 + (level-1) * 2 end },
	level = function(level) return 12 + (level-1)  end,
}
gifts_req5 = {
	stat = { wil=function(level) return 44 + (level-1) * 2 end },
	level = function(level) return 16 + (level-1)  end,
}
gifts_req_high1 = {
	stat = { wil=function(level) return 22 + (level-1) * 2 end },
	level = function(level) return 10 + (level-1)  end,
}
gifts_req_high2 = {
	stat = { wil=function(level) return 30 + (level-1) * 2 end },
	level = function(level) return 14 + (level-1)  end,
}
gifts_req_high3 = {
	stat = { wil=function(level) return 38 + (level-1) * 2 end },
	level = function(level) return 18 + (level-1)  end,
}
gifts_req_high4 = {
	stat = { wil=function(level) return 46 + (level-1) * 2 end },
	level = function(level) return 22 + (level-1)  end,
}
gifts_req_high5 = {
	stat = { wil=function(level) return 54 + (level-1) * 2 end },
	level = function(level) return 26 + (level-1)  end,
}

-- Animal Kinship

newTalentType{ type="wild-gift/animal-kinship", name = "Animal Kinship", generic = false, description = "Empathy and control over the animals of Maj'Eyal." }

applyDarkEmpathy = Talents.main_env.applyDarkEmpathy


newTalent{
	name = "Animal Care",
	type = {"wild-gift/animal-kinship",1},
	require = gifts_req1,
	image = "talents/higher_heal.png",
	points = 5,
	equilibrium = 10,
	is_nature = true,
	cooldown = 50,
	action = function(self, t)
		if not self.animalfriend or self.animalfriend == self then
			self:logCombat(self, "#GREY##Source# has no friend to revive!")
			return nil
		end
		self.animalfriend:removeEffectsFilter({status="detrimental"}, nil, true)
		self.animalfriend.life = self.animalfriend.max_life
		if self.animalfriend.dead then
			local x, y = util.findFreeGrid(self.x, self.y, 5, true, {[Map.ACTOR]=true})
			if not x then
				game.logPlayer(self, "Not enough space to summon!")
				return
			end
			self.animalfriend.dead = nil
			game.zone:addEntity(game.level, self.animalfriend, "actor", x, y)
			game.party:addMember(self.animalfriend, {
				control="full",
				type="minion",
				title="Animal Friend",
				orders = {target=true, leash=true, anchor=true, talents=true, behavior=true, follow=true},
				on_control = function(self)
					self:hotkeyAutoTalents()
				end,
			})
			game.level.map:particleEmitter(x, y, 1, "dark")
		end
		self.animalfriend:setEffect(self.animalfriend.EFF_ALL_STAT, 10, {power=self:mindCrit(self:combatTalentMindDamage(t, 10, 100))/4})
		return true
	end,
	info = function(self, t)
		--local damage = t.getDamage(self, t)
		return ([[Invigorate your animal friend from with a rush of natural energy. If they are dead, this will revive them. They will be empowered, increasing their stats by %d for 10 rounds after being healed completely.]]):
		format(self:combatTalentMindDamage(t, 10, 100)/4)
	end,
}
 
newTalent{
	name = "Befriend Animal",
	type = {"wild-gift/animal-kinship",2},
	require = gifts_req2,
	image = "talents/guardian_of_the_wood.png",
	points = 5,
	equilibrium = 50,
	is_nature = true,
	cooldown = 20,
	range = 6,
	--proj_speed = 20,
	requires_target = true,
	no_npc_use = true,
	direct_hit = function(self, t) if self:getTalentLevel(t) >= 3 then return true else return false end end,
	target = function(self, t)
		local tg = {type="hit", range=self:getTalentRange(t), talent=t}
		return tg
	end,
	getMaxLife = function(self, t) return self:combatTalentLimit(t, 50, 10, 25) end,
	getDamage = function(self, t) return self:combatTalentMindDamage(t, 35, 330) end,
	action = function(self, t)
		if self.summoner then
			self:logCombat(m, "#GREY##Source# cannot control another while they are controlled!")
			return nil
		end
		local tg = self:getTalentTarget(t)
		local x, y = self:getTarget(tg)
		local zeta=game.level.map(x, y, Map.ACTOR)
		if x and y and zeta then
			checker = false
			if zeta.type == "animal" then checker = true end
			if not (self:getTalentLevelRaw(self.T_ZOOLOGY) < 1) then
				if zeta.type == "insect" then checker = true end
				if zeta.type == "vermin" then checker = true end
			end
			if not (self:getTalentLevelRaw(self.T_ZOOLOGY) < 2) then
				if zeta.subtype == "mold" then checker = true end
				if zeta.subtype == "jelly" then checker = true end
			end
			if not (self:getTalentLevelRaw(self.T_ZOOLOGY) < 3) then
				if zeta.type == "aquatic" then checker = true end
			end
			if not (self:getTalentLevelRaw(self.T_ZOOLOGY) < 4) then
				if zeta.subtype == "plants" then checker = true end
			end
			if not (self:getTalentLevelRaw(self.T_ZOOLOGY) < 5) then
				if zeta.type == "dragon" then checker = true end
			end
			if not checker then
				self:logCombat(m, "#GREY##Source# attempts to befriend ".. zeta.name .." but finds they do not respond.")
				return nil
			end
		end

		if not x or not y then return nil end
		self:project(tg, x, y, function(px, py)
			local m = game.level.map(px, py, Map.ACTOR)
			if not m or not m.max_life or not m.life then return end

			local dam = self:mindCrit(t.getDamage(self, t))
			local olddie = rawget(m, "die")
			m.die = function() end
			DamageType:get(DamageType.NATURE).projector(self, px, py, DamageType.NATURE, dam)
			m.die = olddie
			game.level.map:particleEmitter(px, py, 1, "dark")
			if 100 * m.life / m.max_life <= t.getMaxLife(self, t) and self:checkHit(self:combatMindpower()+5, m:combatSpellResist()) and m:canBe("instakill") and m.rank <= 3.2 and not m.summoner and not m.summon_time then
				if game.party and game.party:hasMember(self) then
					for act, def in pairs(game.party.members) do
						if act.summoner and act.summoner == self then
							if act.type == "animal" and act.subtype == "friend" then
								game.party:removeMember(act)
								act:disappear(act)							
							end
						end
					end
				end
				m.type = "animal"
				m.subtype = "friend"
				--m:attr("no_life_regen", 1)
				--m:attr("no_healing", 1)
				m.ai_state.tactic_leash = 100
				m.remove_from_party_on_death = false
				m.no_inventory_access = false
				m.no_party_reward = true
				if m.life_rating < 13 then m.life_rating = 13 end
				if self:getTalentLevel(self.T_ZOOLOGY) > 5 then
					m.life_rating = m.life_rating + math.floor(( self:getTalentLevel(self.T_ZOOLOGY) - 5 ) * 1.4 )
				end
				if m.max_life < ( m.level * 13 ) + 50 then
					m.max_life = ( m.level * 13 ) + 50
				end
				m.body = { INVEN = 1000, QS_MAINHAND = 0, QS_OFFHAND = 0, MAINHAND = 0, OFFHAND = 0, BODY=1, GEM=0 }
				m.life = m.max_life
				m.move_others = true
				m.summoner = self
				m.summoner_gain_exp = true
				m.unused_stats = 0
				m.dead = nil
				--m.undead = 1
				m.no_breath = 1
				m.unused_talents = math.ceil(m.level / 5)
				m.unused_generics = math.ceil(m.level / 5)
				m.silent_levelup = true
				m.clone_on_hit = nil
				m.unused_talents_types = math.ceil(m.level / 20)
				m.level_range = {1, self.max_level}
				m.max_level = self.max_level
				m.exp_worth=0
				m.never_anger = true
				m.save_hotkeys = true
				m.equipdoll="animal"
				m.ai = "tactical"
				m.ai_state = { talent_in=1, ai_move="move_astar", ally_compassion=10 }
				m.ai_tactic = resolvers.tactic"tank"
				m.gainExp = function() end
				m.keep_inven_on_death = true
				m.can_change_level = true
				m.no_points_on_levelup = false
				m.forceLevelup = function(self) 
					if self.summoner then 
						if not ( self.level < self.summoner.level ) then return nil end
						x = self.level
						zeta = mod.class.Actor.forceLevelup(self, self.summoner.level) 
						if x < self.level then
							for z=x + 1,self.level,1 do
								self.unused_stats = self.unused_stats + 1
								if z >= 2 and z % 3 == 0 then self.unused_talents = self.unused_talents + 1 end
								if z >= 2 and z % 3 == 0 then self.unused_generics = self.unused_generics + 1 end
								if z >= 2 and z % math.floor( 10 + ( 3 * ( self.rank - 2 ) ) ) == 0 then self.unused_talents_types = self.unused_talents_types + 1 end
							end
						end
						return zeta
					end 
				end
				if m:knowTalent(m.T_BONE_SHIELD) then m:unlearnTalent(m.T_BONE_SHIELD, m:getTalentLevelRaw(m.T_BONE_SHIELD)) end
				if m:knowTalent(m.T_MULTIPLY) then m:unlearnTalent(m.T_MULTIPLY, m:getTalentLevelRaw(m.T_MULTIPLY)) end
				if m:knowTalent(m.T_SUMMON) then m:unlearnTalent(m.T_SUMMON, m:getTalentLevelRaw(m.T_SUMMON)) end
				m.faction = self.faction
				m.on_act = function(self)
					if game.player ~= self then return end
					if not self.summoner.dead and not self:hasLOS(self.summoner.x, self.summoner.y) then
						if not self:hasEffect(self.EFF_HUSK_OFS) then
							self:setEffect(self.EFF_HUSK_OFS, 3, {})
						end
					else
						if self:hasEffect(self.EFF_HUSK_OFS) then
							self:removeEffect(self.EFF_HUSK_OFS)
						end
					end
				end
				
				self.animalfriend = m
				
				m.on_can_control = function(self, vocal)
					if not self:hasLOS(self.summoner.x, self.summoner.y) then
						if vocal then game.logPlayer(game.player, "Your friend is out of sight; you cannot establish direct control.") end
						return false
					end
					return true
				end

				m:removeEffectsFilter({status="detrimental"}, nil, true)
				game.level.map:particleEmitter(px, py, 1, "demon_teleport")

				game.party:addMember(m, {
					control="full",
					type="minion",
					title="Animal Friend",
					orders = {target=true, leash=true, anchor=true, talents=true, behavior=true, follow=true},
					on_control = function(self)
						self:hotkeyAutoTalents()
					end,
				})

				self:logCombat(m, "#GREY##Source# drains the will to fight from #target# and creates an animal friend.")
			end
		end)

		game:playSoundNear(self, "talents/spell_generic")
		return true
	end,
	info = function(self, t)
		local damage = t.getDamage(self, t)
		return ([[Drain the will to fight out of the target animal, doing %0.2f nature damage (that can never kill the target).
		If the target is left with less than %d%% life you attempt to befriend them.
		Should this succeed the target becomes your permanent minion.
		Animal friends prossess the same abilities as they had in life, and are healed to full.
		Only one animal friend can be had at any time.
		Bosses, and summoned creatures can not be turned into animal friends.
		The damage and chance will increase with your Mindpower.]]):
		format(damDesc(self, DamageType.NATURE, damage), t.getMaxLife(self, t))
	end,
}

newTalent{
	name = "Power of Friendship",
	type = {"wild-gift/animal-kinship",3},
	image = "talents/heal.png",
	require = gifts_req3,
	points = 5,
	is_nature = true,
	equilibrium = 10,
	cooldown = function(self, t) return 10 end,
	remove_on_zero = false,
	getHeal = function(self, t) return self:combatTalentMindDamage(t, 20, 440) end,
	getRegen = function(self, t)
		if not self.animalfriend then
			return 0
		end
		return self.animalfriend.max_life / 10
	end,
	is_heal = true,
	action = function(self, t)
		if not self.animalfriend then
			game.logPlayer(self, "You have no animal friend to draw power from.")
			return nil
		end
		if self.animalfriend.dead then
			game.logPlayer(self, "Your animal friend is defeated and cannot help you!")
			return nil
		end
		game:playSoundNear(self, "talents/spell_generic")
		self:attr("allow_on_heal", 1)
		self:heal(self:mindCrit(t.getHeal(self, t)), self)
		self:attr("allow_on_heal", -1)
		self:setEffect(self.EFF_REGENERATION, 10, {power=self:mindCrit((self.animalfriend.max_life) / 10)})
		game:playSoundNear(self, "talents/heal")
		return true
	end,
	info = function(self, t)
		local heal = t.getHeal(self, t)
		local regen = t.getRegen(self,t)
		return ([[Draw on the strength of your animal friend for  %d life.
		The amount healed will increase with your Mindpower.
		You will also regenerate for 1/10th your animal friend's maximum hit points(%d) per round for ten rounds.]]):
		format(heal,regen)
	end,
}
	newTalent{
	name = "Zoology",
	type = {"wild-gift/animal-kinship",4},
	require = gifts_req4,
	image = "talents/instincts.png",
	points = 5,
	mode = "passive",
	is_nature = true,
	getraise = function(self, t)
		if not ( self:getTalentLevel(self.T_ZOOLOGY) > 5 ) then return 0 end
		return math.floor(( self:getTalentLevel(self.T_ZOOLOGY) - 5 ) * 1.5 )
	end,
	info = function(self, t)
		local raise = t.getraise(self, t)
		return ([[Expands your animal charming abilities to include other types depending on the raw rank of the talent.
		1: Insect/Vermin
		2: Mold/Jelly
		3: Aquatic
		4: Plants
		5: Dragon
		A rating over 5 increases the life rating of any animal friend you create. Currently: %d]]):
		format(raise)
	end,
}

-- Animal Kinship

newTalentType{ type="wild-gift/natural-vengeance", name = "Natural Vengeance", generic = false, description = "The fury of nature itself rises around you, hindering any that would stand in your way." }

newTalentType{ type="wild-gift/animal-training", name = "Animal Training", generic = false, description = "Teach your animal companions new ways of getting things done!" }
newTalentType{ type="wild-gift/kennels", name = "Animal Training", generic = false, description = "Keep your pets safe and stored away." }

newTalent{
	name = "Kenneling",
	type = {"wild-gift/animal-training",1},
	require = gifts_req1,
	image = "talents/absorption_mastery.png",
	points = 5,
	equilibrium = 20.0,
	is_nature = true,
	cooldown = nil,
	no_unlearn_last = true,
	getKennels = function(self, t) return math.floor(self:combatTalentLimit(t, 5, 1, 3)) end, -- Limit 4
	on_learn = function(self, t)
		local lev = t.getKennels(self, t)
		if lev == 1 then
			self:learnTalent(self.T_FIRST_KENNEL, true, nil, {no_unlearn=true})
		elseif lev == 2 then
			self:learnTalent(self.T_SECOND_KENNEL, true, nil, {no_unlearn=true})
		elseif lev == 3 then
			self:learnTalent(self.T_THIRD_KENNEL, true, nil, {no_unlearn=true})
		end
	end,
	on_unlearn = function(self, t)
		local lev = t.getKennels(self, t)
		if lev == 1 then
			self:unlearnTalent(self.T_FIRST_KENNEL)
		elseif lev == 2 then
			self:unlearnTalent(self.T_SECOND_KENNEL)
		elseif lev == 3 then
			self:unlearnTalent(self.T_THIRD_KENNEL)
		end
	end,
	action = function(self, t)
		if not self.animalfriend then
			--local kenchat = Chat.new("kennel", self, self, {npc=self, player=self})
			--kenchat:invoke()
			game.logSeen(self, "#DARK_GREY#You have no animal friend to store.")
			return nil
		end
		themax = t.getKennels(self,t)
		found = 0
		if self.kennel then
			for i=1,themax do
				if not self.kennel[i] then
					found = i
					break
				end
			end
		else 
			found = 1
			self.kennel = {}
		end
		if found == 0 then
			game.logSeen(self, "#DARK_GREY#Your kennels are full.")
			return nil
		end
		self.kennel[found] = self.animalfriend
		game.logSeen(self, "#DARK_GREY#%s sends %s to the kennels.", self.name, self.animalfriend.name)
		game.party:removeMember(self.animalfriend)
		self.animalfriend:disappear(self.animalfriend)		
		self.animalfriend = nil
		return true
	end,
	info = function(self, t)
		local kennels = t.getKennels(self, t)
		return ([[Put your active animal friend away, allowing you to charm a new one without setting the first free.
		Use while you have no animal friend, or while the kennel is full, to draw a friend out of the kennels.
		You can have up to %d stored.]]):
		format(kennels)
	end,
}

newTalent{
	name = "First Kennel",
	type = {"wild-gift/kennels",1},
	require = gifts_req1,
	image = "talents/absorption_mastery.png",
	points = 1,
	no_unlearn_last = true,
	equilibrium = 20.0,
	is_nature = true,
	cooldown = 15,
	action = function(self, t)
		if not self.kennel then
			game.logSeen(self, "#DARK_GREY#You have nothing in your kennels.")
			return nil
		end
		if not self.kennel[1] then
			game.logSeen(self, "#DARK_GREY#You have nothing in that kennel.")
			return nil
		end
		self.tempfriend=self.kennel[1]
		if self.animalfriend then
			self.kennel[1] = self.animalfriend
			game.logSeen(self, "#DARK_GREY#%s sends %s to the kennels.", self.name, self.animalfriend.name)
			game.party:removeMember(self.animalfriend)
			self.animalfriend:disappear(self.animalfriend)
			t.image=self.kennel[1].image
			t.display_entity =  self.kennel[1]
		end
		self.animalfriend = self.tempfriend
		self.tempfriend = nil
		self.animalfriend:removeEffectsFilter({status="detrimental"}, nil, true)
		self.animalfriend.life = self.animalfriend.max_life
		if self.animalfriend.dead then
			local x, y = util.findFreeGrid(self.x, self.y, 5, true, {[Map.ACTOR]=true})
			if not x then
				game.logPlayer(self, "Not enough space to summon!")
				return
			end
			self.animalfriend.dead = nil
			game.zone:addEntity(game.level, self.animalfriend, "actor", x, y)
			game.level.map:particleEmitter(x, y, 1, "dark")
		end
		game.party:addMember(self.animalfriend, {
			control="full",
			type="minion",
			title="Animal Friend",
			orders = {target=true, leash=true, anchor=true, talents=true, behavior=true, follow=true},
			on_control = function(self)
				self:hotkeyAutoTalents()
			end,
		})
		game.logSeen(self, "#DARK_GREY#%s calls %s from the kennels.", self.name, self.animalfriend.name)
		return true
	end,
	info = function(self, t)
		awesometext = [[Allows you to take out your pet in kennel slot 1. If you have a pet out at the moment, they switch places with them.]]
		if self.kennel then 
			if self.kennel[1] then 
				awesometext= awesometext..[[ Currently in the kennel:#GREEN#]]..self.kennel[1].name
			end 
		end
		return awesometext
	end,
}

newTalent{
	name = "Second Kennel",
	type = {"wild-gift/kennels",2},
	require = gifts_req1,
	image = "talents/absorption_mastery.png",
	points = 1,
	no_unlearn_last = true,
	equilibrium = 20.0,
	is_nature = true,
	cooldown = 15,
	action = function(self, t)
		if not self.kennel then
			game.logSeen(self, "#DARK_GREY#You have nothing in your kennels.")
			return nil
		end
		if not self.kennel[2] then
			game.logSeen(self, "#DARK_GREY#You have nothing in that kennel.")
			return nil
		end
		self.tempfriend=self.kennel[2]
		if self.animalfriend then
			self.kennel[2] = self.animalfriend
			game.logSeen(self, "#DARK_GREY#%s sends %s to the kennels.", self.name, self.animalfriend.name)
			game.party:removeMember(self.animalfriend)
			self.animalfriend:disappear(self.animalfriend)
			t.image=self.kennel[2].image
			t.display_entity =  self.kennel[2]
end
		self.animalfriend = self.tempfriend
		self.tempfriend = nil
		self.animalfriend:removeEffectsFilter({status="detrimental"}, nil, true)
		self.animalfriend.life = self.animalfriend.max_life
		if self.animalfriend.dead then
			local x, y = util.findFreeGrid(self.x, self.y, 5, true, {[Map.ACTOR]=true})
			if not x then
				game.logPlayer(self, "Not enough space to summon!")
				return
			end
			self.animalfriend.dead = nil
			game.zone:addEntity(game.level, self.animalfriend, "actor", x, y)
			game.level.map:particleEmitter(x, y, 1, "dark")
		end
		game.party:addMember(self.animalfriend, {
			control="full",
			type="minion",
			title="Animal Friend",
			orders = {target=true, leash=true, anchor=true, talents=true, behavior=true, follow=true},
			on_control = function(self)
				self:hotkeyAutoTalents()
			end,
		})
		game.logSeen(self, "#DARK_GREY#%s calls %s from the kennels.", self.name, self.animalfriend.name)
		return true
	end,
	info = function(self, t)
		awesometext = [[Allows you to take out your pet in kennel slot 2. If you have a pet out at the moment, they switch places with them.]]
		if self.kennel then 
			if self.kennel[2] then 
				awesometext= awesometext..[[ Currently in the kennel:#GREEN#]]..self.kennel[2].name
			end 
		end
		return awesometext
	end,
}

newTalent{
	name = "Third Kennel",
	type = {"wild-gift/kennels",3},
	require = gifts_req1,
	image = "talents/absorption_mastery.png",
	points = 1,
	no_unlearn_last = true,
	equilibrium = 20.0,
	is_nature = true,
	cooldown = 15,
	action = function(self, t)
		if not self.kennel then
			game.logSeen(self, "#DARK_GREY#You have nothing in your kennels.")
			return nil
		end
		if not self.kennel[3] then
			game.logSeen(self, "#DARK_GREY#You have nothing in that kennel.")
			return nil
		end
		self.tempfriend=self.kennel[3]
		if self.animalfriend then
			self.kennel[3] = self.animalfriend
			game.logSeen(self, "#DARK_GREY#%s sends %s to the kennels.", self.name, self.animalfriend.name)
			game.party:removeMember(self.animalfriend)
			self.animalfriend:disappear(self.animalfriend)
			t.image=self.kennel[3].image
			t.display_entity =  self.kennel[3]
		end
		self.animalfriend = self.tempfriend
		self.tempfriend = nil
		self.animalfriend:removeEffectsFilter({status="detrimental"}, nil, true)
		self.animalfriend.life = self.animalfriend.max_life
		if self.animalfriend.dead then
			local x, y = util.findFreeGrid(self.x, self.y, 5, true, {[Map.ACTOR]=true})
			if not x then
				game.logPlayer(self, "Not enough space to summon!")
				return
			end
			self.animalfriend.dead = nil
			game.zone:addEntity(game.level, self.animalfriend, "actor", x, y)
			game.level.map:particleEmitter(x, y, 1, "dark")
		end
		game.party:addMember(self.animalfriend, {
			control="full",
			type="minion",
			title="Animal Friend",
			orders = {target=true, leash=true, anchor=true, talents=true, behavior=true, follow=true},
			on_control = function(self)
				self:hotkeyAutoTalents()
			end,
		})
		game.logSeen(self, "#DARK_GREY#%s calls %s from the kennels.", self.name, self.animalfriend.name)
		return true
	end,
	info = function(self, t)
		awesometext = [[Allows you to take out your pet in kennel slot 3. If you have a pet out at the moment, they switch places with them.]]
		if self.kennel then 
			if self.kennel[3] then 
				awesometext= awesometext..[[ Currently in the kennel:#GREEN#]]..self.kennel[3].name
			end 
		end
		return awesometext
	end,
}

newTalent{
	name = "Fourth Kennel",
	type = {"wild-gift/kennels",4},
	require = gifts_req1,
	image = "talents/absorption_mastery.png",
	points = 1,
	no_unlearn_last = true,
	equilibrium = 20.0,
	is_nature = true,
	cooldown = 15,
	action = function(self, t)
		if not self.kennel then
			game.logSeen(self, "#DARK_GREY#You have nothing in your kennels.")
			return nil
		end
		if not self.kennel[4] then
			game.logSeen(self, "#DARK_GREY#You have nothing in that kennel.")
			return nil
		end
		self.tempfriend=self.kennel[4]
		if self.animalfriend then
			self.kennel[4] = self.animalfriend
			game.logSeen(self, "#DARK_GREY#%s sends %s to the kennels.", self.name, self.animalfriend.name)
			game.party:removeMember(self.animalfriend)
			self.animalfriend:disappear(self.animalfriend)
			--t.image=self.kennel[4].image
			t.display_entity =  self.kennel[4]
		end
		self.animalfriend = self.tempfriend
		self.tempfriend = nil
		self.animalfriend:removeEffectsFilter({status="detrimental"}, nil, true)
		self.animalfriend.life = self.animalfriend.max_life
		if self.animalfriend.dead then
			local x, y = util.findFreeGrid(self.x, self.y, 5, true, {[Map.ACTOR]=true})
			if not x then
				game.logPlayer(self, "Not enough space to summon!")
				return
			end
			self.animalfriend.dead = nil
			game.zone:addEntity(game.level, self.animalfriend, "actor", x, y)
			game.level.map:particleEmitter(x, y, 1, "dark")
		end
		game.party:addMember(self.animalfriend, {
			control="full",
			type="minion",
			title="Animal Friend",
			orders = {target=true, leash=true, anchor=true, talents=true, behavior=true, follow=true},
			on_control = function(self)
				self:hotkeyAutoTalents()
			end,
		})
		game.logSeen(self, "#DARK_GREY#%s calls %s from the kennels.", self.name, self.animalfriend.name)
		return true
	end,
	info = function(self, t)
		awesometext = [[Allows you to take out your pet in kennel slot 4. If you have a pet out at the moment, they switch places with them.]]
		if self.kennel then 
			if self.kennel[4] then 
				awesometext= awesometext..[[ Currently in the kennel:#GREEN#]]..self.kennel[4].name
			end 
		end
		return awesometext
	end,
}

newTalent{
	name = "Take Down Training",
	type = {"wild-gift/animal-training",2},
	require = gifts_req2,
	image = "talents/savagery.png",
	points = 5,
	equilibrium = 50,
	is_nature = true,
	no_unlearn_last = true,
	cooldown = 50,
	getRatio = function(self, t)
		return math.min(math.max(t.getTraining(self,t) * 0.032,1.0),1.6)
	end,
	getTraining = function(self,t)
	 if not self.animalfriend then
	  return 0
	 end
	 return ( self:getTalentLevel(t) * 10 ) / math.max(((self.animalfriend.rank - 2 ) * 2 ) + 1,1)
	end,
	action = function(self, t)
		if not self.animalfriend or self.animalfriend == self then
			self:logCombat(self, "#GREY##Source# has no animal friend to train!")
			return nil
		end
		training = t.getTraining(self, t)
		ratio = t.getRatio(self, t)
		self:logCombat(self, "#GREY##Source# spends some time going over general tricks with their animal friend.")
		self.animalfriend:learnTalentType("technique/combat-techniques-active", false)
		self.animalfriend:setTalentTypeMastery("technique/combat-techniques-active", math.max(ratio,self.animalfriend:getTalentTypeMastery("technique/combat-techniques-active")))
		if training > 9 then
			self.animalfriend:learnTalentType("technique/combat-training", false)
			self.animalfriend:setTalentTypeMastery("technique/combat-training", math.max(ratio,self.animalfriend:getTalentTypeMastery("technique/combat-training")))
		else return true end
		if training > 19 then
			self.animalfriend:learnTalentType("technique/mobility", false)
			self.animalfriend:setTalentTypeMastery("technique/mobility", math.max(ratio,self.animalfriend:getTalentTypeMastery("technique/mobility")))
		else return true end
		if training > 29 then
			self.animalfriend:learnTalentType("technique/superiority", false)
			self.animalfriend:setTalentTypeMastery("technique/superiority", math.max(ratio,self.animalfriend:getTalentTypeMastery("technique/superiority")))
		else return true end
		return true
	end,
	info = function(self, t)
		--local damage = t.getDamage(self, t)
		training = math.min(( t.getTraining(self, t) * 100 ) / 43.75,100)
		ratio=t.getRatio(self, t)
		return ([[Imparts upon your active animal friend new training, allowing them to spend their category points in new, physically oriented, trees. You can use this power more than once, but it will have no effect if you have not improved your rating. The higher your rating, the more varied the training opportunities will be offered to your pet. Higher ranked pets are more difficult to train and will pick up less trees to choose from.
		Training power with current pet: %d%%/%1.1f]]):
		format(training,ratio)
	end,
}

newTalent{
	name = "Together Forever",
	image = "talents/multiply.png",
	type = {"wild-gift/animal-training", 3},
	require = gifts_req2,
	points = 5,
	mode = "sustained",
	sustain_equilibrium = 20,
	remove_on_zero = false,
	cooldown = function(self, t) return 20 end,
	range = 0,
	requires_target = false,
	getPercentage = function(self, t)
		return self:getTalentLevel(t)
	end,
	getClear = function(self, t)
		if self.summoner then
			return self.summoner:getTalentLevel(t) * 2
		else
			return self:getTalentLevel(t) * 2
		end
	end,
	do_statusclear = function(self, t)
		local effs = {}
		-- Go through all spell effects
		for eff_id, p in pairs(self.tmp) do
			local e = self.tempeffect_def[eff_id]
			if e.status == "detrimental" then
				effs[#effs+1] = {"effect", eff_id}
			end
		end
		
		if #effs > 0 then
			local eff = rng.tableRemove(effs)
			if eff[1] == "effect" and rng.percent(t.getClear(self, t)) then
				self:removeEffect(eff[2])
				game.logSeen(self, "%s has recovered with their friend's help!", self.name:capitalize())
			end
		end
	end,
	callbackOnActBase = function(self, t)
		local tt = self:isTalentActive(t.id)
		if not tt then return end
		if not self.animalfriend then
			return nil
		end
		if self.animalfriend.dead then
			return nil
		end
		mine = self.life / self.max_life
		pet = self.animalfriend.life / self.animalfriend.max_life
		animalfriend = self.animalfriend
		animalfriend:incEquilibrium(0 - t.getPercentage(self, t))
		if mine < 0.26 and pet < 0.26 then
			return nil
		end
		if mine > 0.89 and pet > 0.89 then
			return nil
		end
		if mine > pet then
			amount = self.max_life * t.getPercentage(self,t) / 100
			self.life = self.life - amount
			animalfriend:attr("allow_on_heal", 1)
			animalfriend:heal(self:mindCrit(amount*1.5), self)
			animalfriend:attr("allow_on_heal", -1)
			t.do_statusclear(animalfriend, t)
		else
			amount = animalfriend.max_life * t.getPercentage(self,t) / 100
			animalfriend.life = animalfriend.life - amount
			self:attr("allow_on_heal", 1)
			self:heal(animalfriend:mindCrit(amount*1.5), animalfriend)
			self:attr("allow_on_heal", -1)
			t.do_statusclear(self, t)
		end
	end,
	activate = function(self, t)
		return {}
	end,
	deactivate = function(self, t, p)
		return true
	end,

	info = function(self, t)
		return ([[You and your animal friend are in it together to the bloody end. Every round whichever one of you is less injured will give up life energy to renew the other. They will give up %1.1f%% of their hit points and restore half again as much. This will not trigger if the most healthy of the two is at or below 25%%, or if neither is injured beneath 90%%.
		This will also cure 1 negative status with a %d%% chance on the party being healed.
		While active, your animal friend will begin to lose equilibrium at a rate of %d per round.]]):
		format(t.getPercentage(self,t),t.getClear(self,t),t.getPercentage(self,t))
	end,
}

newTalent{
	name = "Nature's Wrath Training",
	type = {"wild-gift/animal-training",4},
	require = gifts_req4,
	image = "talents/beyond_the_flesh.png",
	points = 5,
	equilibrium = 50,
	is_nature = true,
	no_unlearn_last = true,
	cooldown = 50,
	getRatio = function(self, t)
		return math.min(math.max(t.getTraining(self,t) * 0.032,1.0),1.6)
	end,
	getTraining = function(self,t)
	 if not self.animalfriend then
	  return 0
	 end
	 return ( self:getTalentLevel(t) * 10 ) / math.max(((self.animalfriend.rank - 2 ) * 2 ) + 1,1)
	end,
	action = function(self, t)
		if not self.animalfriend or self.animalfriend == self then
			self:logCombat(self, "#GREY##Source# has no animal friend to train!")
			return nil
		end
		training = t.getTraining(self, t)
		ratio = t.getRatio(self, t)
		self:logCombat(self, "#GREY##Source# spends some time going over natural tricks with their animal friend.")
		self.animalfriend:learnTalentType("wild-gift/sand-drake", false)
		self.animalfriend:setTalentTypeMastery("wild-gift/sand-drake", math.max(ratio,self.animalfriend:getTalentTypeMastery("technique/combat-techniques-active")))
		if training > 9 then
			self.animalfriend:learnTalentType("cunning/poisons", false)
			self.animalfriend:setTalentTypeMastery("cunning/poisons", math.max(ratio,self.animalfriend:getTalentTypeMastery("technique/combat-training")))
		else return true end
		if training > 19 then
			self.animalfriend:learnTalentType("psionic/psychic-assault", false)
			self.animalfriend:setTalentTypeMastery("psionic/psychic-assault", math.max(ratio,self.animalfriend:getTalentTypeMastery("technique/mobility")))
		else return true end
		if training > 29 then
			self.animalfriend:learnTalentType("psionic/distortion", false)
			self.animalfriend:setTalentTypeMastery("psionic/distortion", math.max(ratio,self.animalfriend:getTalentTypeMastery("technique/superiority")))
		else return true end
		return true
	end,
	info = function(self, t)
		--local damage = t.getDamage(self, t)
		training = math.min(( t.getTraining(self, t) * 100 ) / 43.75,100)
		ratio=t.getRatio(self, t)
		return ([[Imparts upon your active animal friend new training, allowing them to spend their category points in new, nature oriented, trees. You can use this power more than once, but it will have no effect if you have not improved your rating. The higher your rating, the more varied the training opportunities will be offered to your pet. Higher ranked pets are more difficult to train and will pick up less trees to choose from.
		Training power with current pet: %d%%/%1.1f]]):
		format(training,ratio)
	end,
}

newTalent{
	name = "Grasping Earth",
	type = {"wild-gift/natural-vengeance",1},
	image = "talents/acidic_soil.png",
	mode = "sustained",
	require = gifts_req1,
	points = 5,
	cooldown = 12,
	sustain_equilibrium = 10,
	no_energy = true,
	is_nature = true,
	tactical = { DEBUFF = 2 },
	range = 5,
	getDamage = function(self, t) return self:combatTalentMindDamage(t, 7, 30) end,
	getTargetCount = function(self, t) return math.floor(self:combatTalentScale(t, 1, 10)) end,
	getNegativeDrain = function(self, t) return self:combatTalentLimit(t, 0, 8, 3) end, -- Limit > 0, no regen at high levels
	do_fury = function(self, t)
--		if not self:equilibriumChance() then 
--			game.logSeen(self, "#DARK_GREY#The forces of nature rebel against %s!", self.name)
--			return nil
--		end

		local tgts = {}
		local grids = core.fov.circle_grids(self.x, self.y, 5, true)
		for x, yy in pairs(grids) do for y, _ in pairs(grids[x]) do
			local a = game.level.map(x, y, Map.ACTOR)
			if a and self:reactionToward(a) < 0 then
				tgts[#tgts+1] = a
			end
		end end

		local drain = t.getNegativeDrain(self, t)

		-- Randomly take targets
		local tg = {type="hit", range=self:getTalentRange(t), talent=t}
		for i = 1, t.getTargetCount(self, t) do
			if #tgts <= 0 then break end
			if self:equilibriumChance() then
				local a, id = rng.table(tgts)
				table.remove(tgts, id)

				self:project(tg, a.x, a.y, DamageType.PHYSICAL, rng.avg(t.getDamage(self, t) / 2, self:mindCrit(t.getDamage(self, t)), 3))
				if self:getTalentLevel(self.T_MAGIC_THIRST) > 0 then
					self:project(tg, a.x, a.y, DamageType.MANABURN, rng.avg(t.getDamage(self, t) * self:getTalentLevel(self.T_MAGIC_THIRST) / 20 , t.getDamage(self, t) * self:getTalentLevel(self.T_MAGIC_THIRST) / 10, 3))
					if math.random(1,100) < self:getTalentLevel(self.T_MAGIC_THIRST) * 5 then
						if a:canBe("silence") then
							a:setEffect(a.EFF_SILENCED, 3, {apply_power=self:combatMindpower()})
						end
					end
				end
				if self:getTalentLevel(self.T_PROTECTIVE_ROOTS) > 0 then
					if math.random(1,100) < self:getTalentLevel(self.T_PROTECTIVE_ROOTS) * 5 then
						if a:canBe("stun") then
							a:setEffect(a.EFF_STUNNED, 3, {apply_power=self:combatMindpower()})
						end
					end
				end
				game.level.map:particleEmitter(self.x, self.y, math.max(math.abs(a.x-self.x), math.abs(a.y-self.y)), "breath_earth", {tx=a.x-self.x, ty=a.y-self.y})
				if a:canBe("pin") then
					a:setEffect(a.EFF_PINNED, 3, {apply_power=self:combatMindpower()})
				end
				--local sound = "creatures/treants/treeant_"..math.random(1,3)
				--game:playSoundNear(self, sound)
				self:incEquilibrium(drain)
			end
		end
	end,
	activate = function(self, t)
		game:playSoundNear(self, "talents/spell_generic")
		game.logSeen(self, "#DARK_GREY#The ground begins to churn violently around %s!", self.name)
		return {
		}
	end,
	deactivate = function(self, t, p)
		game.logSeen(self, "#DARK_GREY#The ground around %s calms.", self.name)
		return true
	end,
	info = function(self, t)
		local targetcount = t.getTargetCount(self, t)
		local damage = t.getDamage(self, t)
		local drain = t.getNegativeDrain(self, t)
		return ([[Rouse the fury of nature, causing the plants and even the ground to begin animating in agitation.
		Each turn, up to %d of your foes within radius 5 will be crushed for %d to %d physical damage with a chance to pin in place.
		This powerful gift will raise %0.1f equilibrium with each attack. Attacks will start to falter if you fail equilibrium checks.
		The damage will increase with your Mindpower.]]):
		format(targetcount, damage / 2, damage, drain)
	end,
}

newTalent{
	name = "Magic Thirst",
	type = {"wild-gift/natural-vengeance",2},
	require = gifts_req2,
	image = "talents/instincts.png",
	points = 5,
	equilibrium = 7.0,
	is_nature = true,
	random_ego = "attack",
	cooldown = 6,
	tactical = { ATTACK = { MAGICBURN = 2 } },
	range = function(self, t) 
		if self:getTalentLevelRaw(t) > 4 then
			return math.floor(10 + ( self:getTalentLevel(t) / 3  ))
		end
		return 10
	end,
	direct_hit = function(self, t) if self:getTalentLevel(t) >= 5 then return true else return false end end,
	requires_target = true,
	target = function(self, t)
		local tg = {type="bolt", range=self:getTalentRange(t), talent=t}
		if self:getTalentLevel(t) >= 5 then tg.type = "beam" end
		return tg
	end,
	getDamage = function(self, t) 
		return self:combatTalentMindDamage(t, 25, 100) * ( ( 0.01 * self.level ) + 1  )
	end,
	action = function(self, t)
		local tg = self:getTalentTarget(t)
		local x, y = self:getTarget(tg)
		if not x or not y then return nil end
		self:project(tg, x, y, DamageType.MANABURN, self:mindCrit(t.getDamage(self, t)), nil)
		local _ _, x, y = self:canProject(tg, x, y)
		if tg.type == "beam" then
			game.level.map:particleEmitter(self.x, self.y, math.max(math.abs(x-self.x), math.abs(y-self.y)), "arcanetrail", {tx=x-self.x, ty=y-self.y})
		else
			game.level.map:particleEmitter(x, y, 1, "ball_arcane")
		end
		game:playSoundNear(self, "talents/cloud")
		return true
	end,
	info = function(self, t)
		local damage = t.getDamage(self, t)
		return ([[Send out a lashing of natural energies to leech the foul magics of the target.
		The target will take %0.2f mana burn damage.
		At Talent Level 5, this becomes a piercing line of acid and its range increases based on the total rating of the talent.
		This talent also improves your grasping earth talent, causing an increasing amount of minor mana burn damage and a slight chance to silence to be inflicted to those caught in it.]]
		):format(damDesc(self, DamageType.MANABURN, damage))
	end,
}

newTalent{
	name = "Protective Roots",
	type = {"wild-gift/natural-vengeance",3},
	require = gifts_req3,
	image = "talents/absorption_mastery.png",
	points = 5,
	equilibrium = 12.0,
	is_nature = true,
	random_ego = "defensive",
	cooldown = 15,
	getAbsorb = function(self, t) return self:combatTalentMindDamage(t, 30, 470) end,
	tactical = { DEFEND = 2 },
	action = function(self, t)
		self:setEffect(self.EFF_DAMAGE_SHIELD, 10, {color={0xe1/255, 0xcb/255, 0x3f/255}, power=self:mindCrit(t.getAbsorb(self, t))})
		game:playSoundNear(self, "talents/heal")
		return true
	end,
	info = function(self, t)
		local absorb = t.getAbsorb(self, t)
		return ([[A protective barrier of roots forms around you that lasts for up to 10 turns and negates %d damage.
		The total damage the barrier can absorb will increase with your Mindpower.
		This power also passively causes enemies struck to have a chance of being stunned.]]):
		format(absorb)
	end,
}
